/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  DeadFTP
 *  Copyright (C) 2000-2001 Brandon Lees
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>
#include <glade/glade.h>
#include <time.h>
#include <ctype.h>

#include "globals.h"
#include "main.h"
#include "clipboard.h"

static GtkTargetEntry targets[] = {
  {"STRING", 0, TARGET_STRING},
  {"TEXT", 0, TARGET_TEXT},
  {"COMPOUND_TEXT", 0, TARGET_COMPOUND_TEXT},
};
static int timeout_tag;
char clipboard_text[REGULAR_BUFFER];
char selection_text[REGULAR_BUFFER];
int have_clipboard = 0, have_selection = 0;
int clipboard_next_convert = 0;

static GdkAtom clipboard_atom = GDK_NONE;
static gint clipboard_monitor(gpointer data);

/*
 * init_clipboard_monitor
 * 
 * prepares mainwindow to receive clipboard selections and starts a timer to call clipboard_monitor()
 */
void init_clipboard_monitor()
{
  clipboard_atom = gdk_atom_intern("CLIPBOARD", FALSE);

  gtk_selection_add_targets(deadftp.queue, GDK_SELECTION_PRIMARY, targets,
			    sizeof(targets) / sizeof(GtkTargetEntry));
  gtk_selection_add_targets(deadftp.queue, clipboard_atom, targets,
			    sizeof(targets) / sizeof(GtkTargetEntry));

  /* Do we need to check more than every second? Less?  Find out and change later. */
  timeout_tag = gtk_timeout_add(1000, clipboard_monitor, NULL);
}

/*
 * clipboard_monitor
 *
 * causes a selection_received signal to be emitted with the contents of the clipboard
 */
static gint clipboard_monitor(gpointer data)
{
  if (!deadftp_prefs.selection_monitor) {
    clipboard_next_convert = 0;
    gtk_selection_convert(deadftp.queue, clipboard_atom,
			  gdk_atom_intern("COMPOUND_TEXT", FALSE), GDK_CURRENT_TIME);
  } else if (!clipboard_next_convert) {
    clipboard_next_convert = 1;
    gtk_selection_convert(deadftp.queue, GDK_SELECTION_PRIMARY,
			  gdk_atom_intern("COMPOUND_TEXT", FALSE), GDK_CURRENT_TIME);
  }

  return TRUE;
}

/*
 * parse_url
 *
 * parses url given in curl and stores the values in the other arguments.
 */
int parse_url(char *curl, char *proto, int proto_len, char *username,
	      int username_len, char *password, int password_len, char *hostname,
	      int hostname_len, int *port, char *dir, int dir_len, char *file,
	      int file_len)
{
  char *ch, *p, url[REGULAR_BUFFER], *c, *d;

  g_return_val_if_fail(curl != NULL, 0);

  for (ch = curl, p = url; ch && *ch; ch++) {
    if (*ch == '%') {
      char x = '\0', z;
      z = isdigit(*(ch + 1)) ? (*(ch + 1) - '0') : (*(ch + 1) - 'A') + 10;
      x = (z & 0xf) << 4;
      z = isdigit(*(ch + 1)) ? (*(ch + 1) - '0') : (*(ch + 1) - 'A') + 10;
      x |= z & 0xf;
      *p++ = x;
    } else
      *p++ = *ch;
  }
  *p = '\0';
  ch = strchr(url, ':');
  if (!ch) {
    return 0;
  }
  *ch++ = '\0';
  strncpy(proto, url, proto_len);
  /* file:// URLs only have directory and file but no hostname */
  if (g_strcasecmp(url, "file") == 0) {
    p = strrchr(ch, '/');
    *p++ = '\0';
    /* get rid of extra / */
    if (*(ch + 1) == '/')
      ch++;
    strncpy(dir, ch, dir_len);
    strncpy(file, p, file_len);
    return 1;
  }
  /* Assume that any other URL will have host info */
  while (*ch == '/')
    *ch++ = '\0';
  p = strchr(ch, '@');
  if (p) {
    *p++ = '\0';
    c = strchr(ch, ':');
    if (c) {
      *c++ = '\0';
      strncpy(password, c, password_len);
    }
    strncpy(username, ch, username_len);
    ch = p;
  }
  d = strchr(ch, '/');
  if (!d)
    return 0;
  *d++ = '\0';
  p = strchr(ch, ':');
  if (p) {
    *p++ = '\0';
    *port = atoi(p);
  }
  strncpy(hostname, ch, hostname_len);
  p = strrchr(d, '/');
  if (p) {
    *p++ = '\0';
    *dir = '/';
    strncpy(dir + 1, d, dir_len - 1);
    d = p;
  }
  strncpy(file, d, file_len);
  return 1;
}
