/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  DeadFTP
 *  Copyright (C) 2000-2001 Brandon Lees
 */

#ifndef	_FTP_H_DEAD_
#define	_FTP_H_DEAD_

#include "view.h"
#include "queue.h"
#include "socket.h"

#include <pthread.h>

typedef struct ftp_server {
  /* Host information */
  char hostname[SMALL_BUFFER];
  int port;
  char username[SMALL_BUFFER];
  char password[SMALL_BUFFER];

  /* Directory information */
  char current_directory[REGULAR_BUFFER];
} ftp_server_t;

typedef struct ftp {
  /* Socket containting descriptor for this connection */
  socket_ctx_t *sock;

  /* Host and directory information */
  ftp_server_t server;

  /* State information for server */
  int state;
  unsigned long flags;
  unsigned long quirks;

  /* FTP Queue */
  struct queue_info queue;

  /* Various timeout variables */
  unsigned long timeout;
  unsigned long keepalive;
  int retry;
  int retry_tag;

  /* Several buffers, consider changing how this stuff is done */
  /* extracmd stores unfinished lines between reads.  extracmd_len 
   * is the length of the unfinished line */
  char extracmd[SMALL_BUFFER];
  int extracmd_len;
  /* extradata is used to store unfinished lines when receiving a file list.
   *lextralen is the length of the unfinished line */
  char extradata[SMALL_BUFFER];
  int extralen;

  /* date_start is the column in which the date begins in the file listing
   * from the server.  Date is used since it's the easiest to identify.  
   * The position of the other fields are determined by the position of
   * the date. */
  int date_start;

  /* Data Connection Information */
  socket_ctx_t *datasock;
  unsigned short dataport;
  unsigned long dataip;
  pthread_t data_thread;
  int data_tag;

  /* List of commands.  Head of the list is the current command, rest of list is a queue of commands to send */
  GList *commands;

  /* Message printed by server after login, if the preference is set */
  GList *readme;

  /* Magical variable used by the hostmanager.
   * It's really an xmlNodePtr but it seems like such a waste to
   * include the xml header when it's only used in one file */
  gpointer host_entry;

  /* Time connected to the server */
  unsigned long connect;

  /* Misc variables that may be changed later */
  int gtag;
  char remote_dir[SMALL_BUFFER];
  char scode[5];

  char *rename;
} ftp_t;

/* cause a list after the command has completed */
#define FTP_CMD_FLAG_LIST      (1 << 0)
/* the arguments to the command will be hidden from view in the console */
#define FTP_CMD_FLAG_PASS      (1 << 1)
/* The command has been sent to the server */
#define FTP_CMD_FLAG_SENT      (1 << 2)

#define FTP_CMD_FLAG_SET(x, flag)	(((command_t *) (x))->flags |= (flag))
#define FTP_CMD_FLAG_CLEAR(x, flag)	(((command_t *) (x))->flags &= ~(flag))
#define FTP_CMD_FLAG_IS_SET(x, flag)	(((command_t *) (x))->flags & (flag))

#define FTP_COMMAND()          ((command_t *) FTP.commands->data)

typedef struct command {
  char *command;
  unsigned long flags;
  int state;
  char *status;
} command_t;

typedef int rcode_t[3];
typedef gboolean(*parse_func_t) (rcode_t reply, const char *arg);
typedef struct {
  parse_func_t handler;
  /* not sure if I want anything else in here */
} parse_desc_t;

/* This IS_SET allows for x values that are |'s of multiple flags.
 * anding will make the value only the falgs in x and xoring will make the value
 * zero if all of the flags are set.  It is then !'d to make the zero value true
 */
#define	IS_SET(x)		(!((FTP.flags & (x)) ^ (x)))
#define	SET_FLAG(x)		(FTP.flags |= (x))
#define	CLEAR_FLAG(x)		(FTP.flags &= ~(x))

#define	FTP_FLAG_START_TRANS	 (1 << 0)
#define	FTP_FLAG_CHANGE_DIR	 (1 << 1)
#define	FTP_FLAG_THREAD_DONE	 (1 << 2)
#define	FTP_FLAG_MULTI		 (1 << 3)
#define	FTP_FLAG_ADD_LIST_QUEUE	 (1 << 4)
#define	FTP_FLAG_CHANGE_DIR_BACK (1 << 5)
#define	FTP_FLAG_HAVE_README	 (1 << 6)
#define	FTP_FLAG_NEW_LIST_DIR	 (1 << 7)
#define	FTP_FLAG_RECURSIVE_LIST  (1 << 8)
#define	FTP_FLAG_FREEZE_QUEUE	 (1 << 9)
#define	FTP_FLAG_ABORT		 (1 << 10)
#define	FTP_FLAG_PASV		 (1 << 11)
#define	FTP_FLAG_TRANS_CWD	 (1 << 12)
#define	FTP_FLAG_DONT_TRANSFER	 (1 << 13)
#define	FTP_FLAG_FOUND_DATE	 (1 << 14)
#define	FTP_FLAG_TRANSFER_END	 (1 << 15)
#define FTP_FLAG_REST_SUPPORT    (1 << 16)
/* For the transfer to be complete, the thread must be done and we must have received
 * the proper code from the server
 */
#define	FTP_FLAG_TRANSFER_DONE	(FTP_FLAG_THREAD_DONE|FTP_FLAG_TRANSFER_END)

extern ftp_t FTP;
extern GList *remote_row;

void ftp_init(void);
void ftp_disconnect(void);
void ftp_connect(void);
void ftp_exit(int status);
void ftp_send_command(int state, unsigned long flags, const char *stat_message,
		      const char *command, ...);
void ftp_mkdir(void);
void ftp_remove(void);
void ftp_rename(void);
void ftp_add_queue(void);
void ftp_refresh(void);
void init_remote_dir_layout(void);
void ftp_do_rename(const char *, const char *);
void ftp_change_remote_dir(const char *dir);
void ftp_abort(void);
void ftp_add_dir(char *);
void ftp_disconnect_retry();

void ftp_history_add_item(GList ** history, char *value);
void ftp_history_set_list(GList * history, GtkWidget * combo);
void ftp_history_save_list(GList * history, const char *section);
void ftp_history_load_list(GList ** history, const char *section);

void ftp_start_data_connection(int state);
void ftp_clear_commands(void);

typedef enum { DATA_THREAD, LIST_THREAD } thread_type;
void create_data_thread(thread_type type, void *(*function) (void *), void *arg);
void stop_data_thread(void);

gint ftp_timeout_data(gpointer data);
gint ftp_timeout_list(gpointer data);

#define FTP_STATE_DISCONNECTED  -1
#define FTP_STATE_CONNECTING    0
#define	FTP_STATE_INIT		1
#define	FTP_STATE_SENT_USER	2
#define	FTP_STATE_SENT_PASS	3
#define	FTP_STATE_NEED_LIST	4
#define	FTP_STATE_CWD		5
#define	FTP_STATE_START_LIST	6
#define	FTP_STATE_TRANS_CWD	7
#define	FTP_STATE_TRANS_SIZE	8
#define	FTP_STATE_TRANS_REST	9
#define	FTP_STATE_TRANS_PASV	10
#define	FTP_STATE_TRANS_RETR	11
#define	FTP_STATE_TRANS_TYPE	12
#define	FTP_STATE_IDLE		13
#define	FTP_STATE_DOING_LIST	14
#define	FTP_STATE_MKDIR		15
#define	FTP_STATE_DELETE	16
#define	FTP_STATE_RENAME_FROM	17
#define	FTP_STATE_RENAME_TO	18
#define	FTP_STATE_ABORT		19
#define	FTP_STATE_FXP_SIZE      20
#define FTP_STATE_FXP_PASV      21
#define FTP_STATE_FXP_PORT      22
#define FTP_STATE_FXP_TRANS     23
#define FTP_STATE_REST_TEST     24
#define FTP_STATE_REST_DONE     25
#define FTP_STATE_CHMOD         26
#define FTP_STATE_D_TRANS_MKD   27

#define	FTP_KEEPALIVE		250

#define FTP_RCODE_REPLY		0
#define FTP_RCODE_REASON	1
#define FTP_RCODE_DETAIL	2

#define	FTP_REPLY_OK_WAIT	1
#define	FTP_REPLY_OK		2
#define	FTP_REPLY_OK_MORE	3
#define	FTP_REPLY_AGAIN		4
#define	FTP_REPLY_BAD		5

#define	FTP_IO_TIMEOUT		2	/* How long should recv/send block?  In seconds */

#define	FTP_HISTORY_SAVE	10

#define	FTP_QUIRK_CLEAR()	(FTP.quirks = 0)
#define	FTP_QUIRK_ADD(q)	(FTP.quirks |= (q))
#define	FTP_QUIRK(q)		(FTP.quirks & (q))

/* Server quirks */
#define	FTP_QUIRK_RECURSIVE_L	(1 << 0)
#define FTP_QUIRK_MICROSOFT     (1 << 1)

typedef struct {
  char *server_string;		/* Identifying string */
  unsigned long quirks;
} quirk_t;

#endif /* _FTP_H_DEAD_ */
