/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  DeadFTP
 *  Copyright (C) 2000-2001 Brandon Lees
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>
#include <glade/glade.h>

#include "globals.h"
#include "main.h"
#include "popups.h"
#include "local.h"
#include "ftp.h"
#include "clipboard.h"

/* Local functions */
gint show_popup(GtkWidget * widget, GdkEvent * e);
gint local_show_popup(GtkWidget * widget, GdkEvent * e);
gint remote_show_popup(GtkWidget * widget, GdkEvent * e);

gint show_popup(GtkWidget * widget, GdkEvent * e)
{
  GdkEventButton *event = (GdkEventButton *) e;

  if ((event->type == GDK_BUTTON_PRESS) && (event->button == 3)) {
    gtk_menu_popup(GTK_MENU(widget), NULL, NULL, NULL, NULL, event->button,
		   event->time);
    return TRUE;
  }
  return FALSE;
}

gint local_show_popup(GtkWidget * widget, GdkEvent * e)
{
  GdkEventButton *event = (GdkEventButton *) e;

  if ((event->type == GDK_BUTTON_PRESS) && (event->button == 3)) {
    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM
				   (deadftp.local_popup.
				    arrange[deadftp_prefs.local_sort_field]), TRUE);
    if (deadftp_prefs.local_sort_order == DIR_SORT_ASC)
      gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM
				     (deadftp.local_popup.arrange_ascending), TRUE);
    else
      gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM
				     (deadftp.local_popup.arrange_descending), TRUE);
    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM
				   (deadftp.local_popup.arrange_autosize),
				   deadftp_prefs.local_auto_size);

    gtk_menu_popup(GTK_MENU(widget), NULL, NULL, NULL, NULL, event->button,
		   event->time);
    return TRUE;
  }
  return FALSE;
}

gint remote_show_popup(GtkWidget * widget, GdkEvent * e)
{
  GdkEventButton *event = (GdkEventButton *) e;

  if ((event->type == GDK_BUTTON_PRESS) && (event->button == 3)) {
    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM
				   (deadftp.remote_popup.
				    arrange[deadftp_prefs.remote_sort_field]), TRUE);
    if (deadftp_prefs.remote_sort_order == DIR_SORT_ASC)
      gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM
				     (deadftp.remote_popup.arrange_ascending), TRUE);
    else
      gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM
				     (deadftp.remote_popup.arrange_descending),
				     TRUE);
    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM
				   (deadftp.remote_popup.arrange_autosize),
				   deadftp_prefs.remote_auto_size);

    gtk_menu_popup(GTK_MENU(widget), NULL, NULL, NULL, NULL, event->button,
		   event->time);
    return TRUE;
  }
  return FALSE;
}


GtkTargetEntry queue_targets[] = {
  {"xml/file-info", 0, 0}
  ,
  {"text/plain", 0, TARGET_STRING}
  ,
  {"_NETSCAPE_URL", 0, TARGET_URL}
  ,
  {"text/unicode", 0, 0}
  ,
  {"xml/fxp-info", 0, 0}
  ,
  {"gtk-clist-drag-reorder", GTK_TARGET_SAME_WIDGET, 0}
  ,
};

void init_popups()
{
  GtkWidget *menu;
  GladeXML *xml;

  xml = glade_xml_new(deadftp.gladefile, "local_popup");
  if (!xml) {
    g_error(_("Couldn't load the interface!"));
    return;
  }
  glade_xml_signal_autoconnect(xml);
  menu = glade_xml_get_widget(xml, "local_popup");
  deadftp.local_popup.arrange[DIR_LAYOUT_FILE] =
    glade_xml_get_widget(xml, "by_name2");
  deadftp.local_popup.arrange[DIR_LAYOUT_LENGTH] =
    glade_xml_get_widget(xml, "by_length2");
  deadftp.local_popup.arrange[DIR_LAYOUT_MODE] =
    glade_xml_get_widget(xml, "by_attributes2");
  deadftp.local_popup.arrange[DIR_LAYOUT_OWNER] =
    glade_xml_get_widget(xml, "by_owner2");
  deadftp.local_popup.arrange[DIR_LAYOUT_GROUP] =
    glade_xml_get_widget(xml, "by_group2");
  deadftp.local_popup.arrange[DIR_LAYOUT_DATE] =
    glade_xml_get_widget(xml, "by_date2");
  deadftp.local_popup.arrange_ascending = glade_xml_get_widget(xml, "ascending2");
  deadftp.local_popup.arrange_descending = glade_xml_get_widget(xml, "descending2");
  deadftp.local_popup.arrange_autosize = glade_xml_get_widget(xml, "auto_size2");
  /* setup the inital values for the popup */
  gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM
				 (deadftp.local_popup.
				  arrange[deadftp_prefs.local_sort_field]), TRUE);
  if (deadftp_prefs.local_sort_order == DIR_SORT_ASC)
    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM
				   (deadftp.local_popup.arrange_ascending), TRUE);
  else
    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM
				   (deadftp.local_popup.arrange_descending), TRUE);
  gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM
				 (deadftp.local_popup.arrange_autosize),
				 deadftp_prefs.local_auto_size);
  gtk_signal_connect_object(GTK_OBJECT(deadftp.local), "button-press-event",
			    GTK_SIGNAL_FUNC(local_show_popup), GTK_OBJECT(menu));
  gtk_object_unref(GTK_OBJECT(xml));

  gtk_drag_source_set(deadftp.local, GDK_BUTTON1_MASK | GDK_BUTTON3_MASK,
		      queue_targets, 4, GDK_ACTION_COPY | GDK_ACTION_DEFAULT);

  xml = glade_xml_new(deadftp.gladefile, "remote_popup");
  if (!xml) {
    g_error(_("Couldn't load the interface!"));
    return;
  }
  glade_xml_signal_autoconnect(xml);
  menu = glade_xml_get_widget(xml, "remote_popup");
  deadftp.remote_popup.arrange[DIR_LAYOUT_FILE] =
    glade_xml_get_widget(xml, "by_name1");
  deadftp.remote_popup.arrange[DIR_LAYOUT_LENGTH] =
    glade_xml_get_widget(xml, "by_length1");
  deadftp.remote_popup.arrange[DIR_LAYOUT_MODE] =
    glade_xml_get_widget(xml, "by_attributes1");
  deadftp.remote_popup.arrange[DIR_LAYOUT_OWNER] =
    glade_xml_get_widget(xml, "by_owner1");
  deadftp.remote_popup.arrange[DIR_LAYOUT_GROUP] =
    glade_xml_get_widget(xml, "by_group1");
  deadftp.remote_popup.arrange[DIR_LAYOUT_DATE] =
    glade_xml_get_widget(xml, "by_date1");
  deadftp.remote_popup.arrange_ascending = glade_xml_get_widget(xml, "ascending1");
  deadftp.remote_popup.arrange_descending = glade_xml_get_widget(xml, "descending1");
  deadftp.remote_popup.arrange_autosize = glade_xml_get_widget(xml, "auto_size1");
  gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM
				 (deadftp.remote_popup.
				  arrange[deadftp_prefs.remote_sort_field]), TRUE);
  if (deadftp_prefs.remote_sort_order == DIR_SORT_ASC)
    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM
				   (deadftp.remote_popup.arrange_ascending), TRUE);
  else
    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM
				   (deadftp.remote_popup.arrange_descending), TRUE);
  gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM
				 (deadftp.remote_popup.arrange_autosize),
				 deadftp_prefs.remote_auto_size);
  gtk_signal_connect_object(GTK_OBJECT(deadftp.remote), "button-press-event",
			    GTK_SIGNAL_FUNC(remote_show_popup), GTK_OBJECT(menu));

  gtk_drag_source_set(deadftp.remote, GDK_BUTTON1_MASK | GDK_BUTTON3_MASK,
		      queue_targets, 5, GDK_ACTION_COPY);

  xml = glade_xml_new(deadftp.gladefile, "queue_popup");
  if (!xml) {
    g_error(_("Couldn't load the interface!"));
    return;
  }
  glade_xml_signal_autoconnect(xml);
  menu = glade_xml_get_widget(xml, "queue_popup");
  gtk_signal_connect_object(GTK_OBJECT(deadftp.queue), "button-press-event",
			    GTK_SIGNAL_FUNC(show_popup), GTK_OBJECT(menu));

  /* dag_dest is already set from calling gtk_clist_set_reorderable so first
   * unset it.  When we reset it we include the reorderable targets so reordering
   * will still work */
  gtk_drag_dest_unset(deadftp.queue);
  gtk_drag_dest_set(deadftp.queue,
		    GTK_DEST_DEFAULT_MOTION | GTK_DEST_DEFAULT_HIGHLIGHT,
		    queue_targets, 6, GDK_ACTION_COPY | GDK_ACTION_MOVE);

}
